//
//  CoordImage.java
//  Affichage d'une image avec des coordonnes
//

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.image.*;

public class CoordImage extends Applet {

    ImageCanvas ic;
    Label lab;
    
    public void init() {
        setLayout(new BorderLayout());
        String nomImage = getParameter("image");
        String sx1 = getParameter("x1");
        String sx2 = getParameter("x2");
        String sy1 = getParameter("y1");
        String sy2 = getParameter("y2");
        String labelx = getParameter("labelx");
        if (sx1 == null || sx2 == null)
            labelx = null;
        else if (labelx == null)
            labelx = "x";
        String labely = getParameter("labely");
        if (sy1 == null || sy2 == null)
            labely = null;
        else if (labely == null)
            labely = "y";
        if  (nomImage == null || ((sx1 == null || sx2 == null) && (sy1 == null || sy2 == null))) {
            System.err.println("Il manque un des paramtres de l'applet");
            return;
        }
        double x1,x2,y1,y2;
        try {
            if (labelx == null)
                x1 = x2 = 0;
            else {
                x1 = (new Double(sx1)).doubleValue();
                x2 = (new Double(sx2)).doubleValue();
            }
            if (labely == null)
                y1 = y2 = 0;
            else {
                y1 = (new Double(sy1)).doubleValue();
                y2 = (new Double(sy2)).doubleValue();
            }
        } catch (NumberFormatException ex) {
            System.err.println("Coordonnes mal formattes.");
            return;
        }
        Image im = getImage(getCodeBase(), nomImage);
        ic = new ImageCanvas(im, this, 200, 200, x1, x2, y1, y2, labelx, labely);
        ic.addMouseMotionListener(ic);
        add(ic, BorderLayout.CENTER);
        lab = new Label(" ");
        add(lab, BorderLayout.SOUTH);
        validate();
    }
    
    class ImageCanvas extends Canvas implements MouseMotionListener {
        Container pappy;
        Image image;
        Dimension size;
        int w, h;
        boolean trueSizeKnown=false;
        MediaTracker tracker;
        int wleft = 60;
        int hbottom = 50;
        double x1,x2,y1,y2;
        String labelx, labely;
    
        public ImageCanvas(Image image, Container highestContainer, 
                        int initialWidth, int initialHeight,
                        double x1, double x2, double y1, double y2,
                        String labelx, String labely) {
            
            if (image == null) {
                System.err.println("Canvas got invalid image object!");
                return;
            }
    
            this.image = image;
            this.pappy = highestContainer;
            
            this.x1 = x1;
            this.x2 = x2;
            this.y1 = y1;
            this.y2 = y2;
    
            this.labelx = labelx;
            this.labely = labely;
            
            w = initialWidth;
            h = initialHeight;
    
            tracker = new MediaTracker(this);
            tracker.addImage(image, 0);
    
            size = new Dimension(w,h);

            setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
        }
    
        public Dimension getPreferredSize() {
            return getMinimumSize();
        }
    
        public Dimension getMinimumSize() {
            return size;
        }
    
        public void paint (Graphics g) {
            if (image != null) {
                if (!trueSizeKnown) {
                    int imageWidth = image.getWidth(this);
                    int imageHeight = image.getHeight(this);
    
                    if (tracker.checkAll(true)) {
                        trueSizeKnown = true;
                        if (tracker.isErrorAny()) {
                            System.err.println("Error loading the image");
                        }
                    }
    
                    //Component-initiated resizing.
                    if (((imageWidth > 0) && (w != imageWidth + wleft)) ||
                        ((imageHeight > 0) && (h != imageHeight + hbottom))) {
                        w = imageWidth + wleft;
                        h = imageHeight + hbottom;
                        size = new Dimension(w,h);
                        setSize(w, h);
                        pappy.validate();
                    }
                }
            }
    
            g.drawImage(image, wleft, 0, this);
            g.drawRect(wleft, 0, w - wleft - 1, h - hbottom - 1);
            
            Font defonce = g.getFont();
            Font plafond = new Font(defonce.getName(), Font.PLAIN, 10);
            g.setFont(plafond);
            Font boldfont = new Font(defonce.getName(), Font.BOLD, 10);
            
            // Axe x
            
            if (labelx != null) {
                double facteurx;
                if (Math.abs(x2 - x1) < 0.15)
                    facteurx = 100; // affiche les centimes
                else if (Math.abs(x2 - x1) < 1.5)
                    facteurx = 10; // affiche les diximes
                else if (Math.abs(x2 - x1) < 15)
                    facteurx = 1;
                else if (Math.abs(x2 - x1) < 150)
                    facteurx = 0.1;
                else
                    facteurx = 0.01;
                
                grilleX(g, facteurx);
                
                g.setFont(boldfont);
                String sx = labelx;
                g.drawString(sx, wleft + (w - wleft)/2 - (g.getFontMetrics().stringWidth(sx))/2, h - hbottom/4);
                g.setFont(plafond);
            }
            
            // Axe y
            
            if (labely != null) {
                double facteury;
                if (Math.abs(y2 - y1) < 0.11)
                    facteury = 100; // affiche les centimes
                else if (Math.abs(y2 - y1) < 1.1)
                    facteury = 10; // affiche les diximes
                else if (Math.abs(y2 - y1) < 11)
                    facteury = 1;
                else if (Math.abs(y2 - y1) < 110)
                    facteury = 0.1;
                else
                    facteury = 0.01;
                
                grilleY(g, facteury);
                
                g.setFont(boldfont);
                String sy = labely;
                g.drawString(sy, 2, (h - hbottom)/2);
                g.setFont(plafond);
            }

        }
        
        public void grilleX(Graphics g, double facteur) {
            int ix1, ix2, px;
            String scoord;
            if (x2 > x1) {
                ix1 = (int)Math.ceil(x1*facteur);
                ix2 = (int)Math.floor(x2*facteur);
            } else {
                ix1 = (int)Math.ceil(x2*facteur);
                ix2 = (int)Math.floor(x1*facteur);
            }
            for (int i=ix1; i<=ix2; i++) {
                if (x2 > x1)
                    px = (int)Math.rint(((i/facteur - x1) / (x2 - x1)) * (w - wleft)) + wleft;
                else
                    px = w - (int)Math.rint(((i/facteur - x2) / (x1 - x2)) * (w - wleft));
                scoord = joliFormattage(i/facteur, 3);
                g.drawString(scoord, px - (g.getFontMetrics().stringWidth(scoord))/2, h - hbottom/2);
                g.setXORMode(Color.white);
                g.drawLine(px, 0, px, h - hbottom);
                g.setPaintMode();
            }
        }
        
        public void grilleY(Graphics g, double facteur) {
            int iy1, iy2, py;
            String scoord;
            if (y2 > y1) {
                iy1 = (int)Math.ceil(y1*facteur);
                iy2 = (int)Math.floor(y2*facteur);
            } else {
                iy1 = (int)Math.ceil(y2*facteur);
                iy2 = (int)Math.floor(y1*facteur);
            }
            for (int i=iy1; i<=iy2; i++) {
                if (y2 > y1)
                    py = (int)Math.rint(((i/facteur - y1) / (y2 - y1)) * (h - hbottom));
                else
                    py = h - hbottom - (int)Math.rint(((i/facteur - y2) / (y1 - y2)) * (h - hbottom));
                scoord = joliFormattage(i/facteur, 3);
                g.drawString(scoord, 10, py);
                g.setXORMode(Color.white);
                g.drawLine(wleft, py, w, py);
                g.setPaintMode();
            }
        }
        
        public void mouseMoved(MouseEvent e) {
            if (e.getX() < w && e.getX() > wleft && e.getY() < h - hbottom) {
                double x = ((e.getX() - wleft) * 1.0 / (w - wleft)) * (x2 - x1) + x1;
                double y = (e.getY() * 1.0 / (h - hbottom)) * (y2 - y1) + y1;
                String sx = joliFormattage(x, 5);
                String sy = joliFormattage(y, 5);
                if (labelx != null && labely != null)
                    lab.setText(" " + sx + ", " + sy);
                else if (labelx != null)
                    lab.setText(" " + sx);
                else if (labely != null)
                    lab.setText(" " + sy);
            } else
                lab.setText(" ");
        }
        
        public void mouseDragged(MouseEvent e) {
        }
        
    }

	// mthode d'affichage de double issue du Calcotron
    public static String joliFormattage(double v, int precision) {
        int limitesci = 3; // dcimales avant de passer en mode sci
        
        if (v == 0)
            return("0");
        String sj = ""+v;
        if ("Infinity".equals(sj) || "-Infinity".equals(sj) || "NaN".equals(sj))
            return(""+v);
        String signe;
        if (v >= 0)
            signe = "";
        else {
            v = -v;
            signe = "-";
        }
        int exposant = (int)Math.floor(Math.log(v)/Math.log(10));
        double exparr = Math.pow(10, precision);
        double mantisse = v / Math.pow(10, exposant);
        mantisse = Math.rint(mantisse*exparr)/exparr;
        if (mantisse >= 10) {
            exposant += 1;
            mantisse = v / Math.pow(10, exposant);
            mantisse = Math.rint(mantisse*exparr)/exparr;
        }
        String sv = "";
        int chiffre = (int)Math.floor(mantisse);
        sv += chiffre + ".";
        for (int i=0; i<precision-1; i++) {
            mantisse = (mantisse - chiffre)*10;
            double exparr2 = Math.pow(10, precision-i);
            mantisse = Math.rint(mantisse*exparr)/exparr;
            chiffre = (int)Math.floor(mantisse);
            sv += chiffre;
        }
        mantisse = (mantisse - chiffre)*10;
        chiffre = (int)Math.round(mantisse);
        sv += chiffre;
        char dernier = sv.charAt(sv.length()-1);
        while (dernier == '0') {
            sv = sv.substring(0, sv.length()-1);
            dernier = sv.charAt(sv.length()-1);
        }
        if (exposant != 0) {
            if (exposant > 0 && exposant < limitesci) {
                String sd = sv.substring(0,1);
                if (sv.length()-2 < exposant) {
                    // transformation de 3.2E2 en 320
                    if (sv.length() > 2)
                        sd += sv.substring(2,sv.length());
                    for (int i=0; i<exposant-(sv.length()-2); i++)
                        sd += '0';
                } else {
                    // transformation de 3.245E2 en 324.5
                    if (sv.length() < exposant+2)
                        sd += sv.substring(2);
                    else
                        sd += sv.substring(2,exposant+2) + "." + sv.substring(exposant+2);
                }
                sv = sd;
            } else if (exposant < 0 && exposant > -limitesci) {
                // transformation de 3.2E-2 en 0.032
                String sd = "0.";
                for (int i=0; i<-exposant-1; i++)
                    sd += '0';
                sv = sd + sv.charAt(0) + sv.substring(2);
            } else {
                if (!sv.endsWith("."))
                    sv += ".";
                sv += "10^" + exposant;
            }
        }
        if (sv.charAt(sv.length()-1) == '.')
            sv = sv.substring(0, sv.length()-1);
        
        sv = signe + sv;
        
        return(sv);
    }
}
