/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

var image;
var canvas;
var context;
var wleft = 50;
var wright = 20;
var hbottom = 50;
var w, h; // dimensions du canvas
var x1, x2, y1, y2; // coordonnées sous forme de nombres
var labelx, labely; // titres des axes

function lireParametres() {
    var parametres = new Array();
    var searchString = window.location.search.substring(1);
    var params = searchString.split("&");
    
    for (var i=0; i<params.length; i++) {
        var nomval = params[i].split("=");
        // remarque: les paramètres de l'URL doivent être encodés en UNICODE
        parametres[nomval[0]] = decodeURIComponent(nomval[1]);
    }
    return(parametres);
}

function init() {
    var parametres = lireParametres();
    var chemin_image = parametres['image'];
    var sx1 = parametres['x1'];
    var sx2 = parametres['x2'];
    var sy1 = parametres['y1'];
    var sy2 = parametres['y2'];
    labelx = parametres['labelx'];
    if (sx1 == null || sx2 == null)
        labelx = null;
    else if (labelx == null)
        labelx = 'x';
    labely = parametres['labely'];
    if (sy1 == null || sy2 == null)
        labely = null;
    else if (labely == null)
        labely = 'y';
    if (chemin_image == null || sx1 == null || sx2 == null || sy1 == null || sy2 == null) {
        alert("Paramètres obligatoires: image, x1, x2, y1, y2; facultatifs: labelx, labely");
        return;
    }
    var chemin_fichiers = parametres['chemin_fichiers'];
    if (chemin_fichiers != null)
        chemin_image = chemin_fichiers + chemin_image;
    x1 = Number(sx1);
    x2 = Number(sx2);
    y1 = Number(sy1);
    y2 = Number(sy2);
    image = new Image();
    image.onload = premierAffichage;
    image.onerror = function(err) { alert("Erreur au chargement de l'image " + chemin_image) };
    image.src = chemin_image;
}

function premierAffichage() {
    var body = document.getElementsByTagName('body')[0];
    canvas = document.createElement('canvas');
    w = image.width + wleft + wright;
    canvas.setAttribute('width', w);
    h = image.height + hbottom;
    canvas.setAttribute('height', h);
    body.appendChild(canvas);
    if (typeof G_vmlCanvasManager  != 'undefined' ) {
        G_vmlCanvasManager.initElement(canvas);
    }
    if (canvas.addEventListener)
        canvas.addEventListener('mousemove', mousemove, false);
    else
        canvas.attachEvent('onmousemove', mousemove);
    if ('ontouchstart' in document) {
        canvas.addEventListener('touchstart', mousemove, false);
        canvas.addEventListener('touchmove', mousemove, false);
    }
    
    if (!canvas.getContext) {
        body.innerHTML = '<p>Votre navigateur ne gère pas HTML5.</p>';
        return;
    }
    context = canvas.getContext('2d');
    paint();
}

function paint() {
    context.drawImage(image, wleft, 0, image.width, image.height);
    
    context.font = 'normal 10px sans-serif'; // il faut ajouter normal pour éviter un bug de FlashCanvas
    context.textAlign = 'left';
    context.textBaseline = 'alphabetic';
    context.fillStyle = 'black';
    
    // Axe x
    
    if (labelx != null) {
        var facteurx;
        var base = image.width / 30;
        if (base > 10)
            base = 10;
        if (Math.abs(x2 - x1) < base * 0.01)
            facteurx = 100; // affiche les centièmes
        else if (Math.abs(x2 - x1) < base * 0.05)
            facteurx = 20; // affiche les moitiés de dixièmes
        else if (Math.abs(x2 - x1) < base * 0.1)
            facteurx = 10; // affiche les dixièmes
        else if (Math.abs(x2 - x1) < base * 0.5)
            facteurx = 2;
        else if (Math.abs(x2 - x1) < base)
            facteurx = 1;
        else if (Math.abs(x2 - x1) < base * 5)
            facteurx = 0.2;
        else if (Math.abs(x2 - x1) < base * 10)
            facteurx = 0.1;
        else if (Math.abs(x2 - x1) < base * 50)
            facteurx = 0.02;
        else
            facteurx = 0.01;
        
        grilleX(facteurx);
        
        context.font = 'bold 10px sans-serif';
        var metrics = context.measureText(labelx);
        context.fillText(labelx, wleft + image.width/2 - metrics.width/2, h - hbottom/4);
        context.font = 'normal 10px sans-serif';
    }
    
    // Axe y
    
    if (labely != null) {
        var facteury;
        var base = image.height / 15;
        if (base > 10)
            base = 10;
        if (Math.abs(y2 - y1) < base * 0.01)
            facteury = 100; // affiche les centièmes
        else if (Math.abs(y2 - y1) < base * 0.05)
            facteury = 20; // affiche les moitiés de dixièmes
        else if (Math.abs(y2 - y1) < base * 0.1)
            facteury = 10; // affiche les dixièmes
        else if (Math.abs(y2 - y1) < base * 0.5)
            facteury = 2;
        else if (Math.abs(y2 - y1) < base)
            facteury = 1;
        else if (Math.abs(y2 - y1) < base * 5)
            facteury = 0.2;
        else if (Math.abs(y2 - y1) < base * 10)
            facteury = 0.1;
        else if (Math.abs(y2 - y1) < base * 50)
            facteury = 0.02;
        else
            facteury = 0.01;
        
        grilleY(facteury);
        
        context.font = 'bold 10px sans-serif';
        context.fillText(labely, 2, (h - hbottom)/2);
        context.font = 'normal 10px sans-serif';
    }
    
}

function grilleX(facteur) {
    var ix1, ix2, px;
    var scoord;
    
    context.save();
    if (x2 > x1) {
        ix1 = Math.ceil(x1*facteur);
        ix2 = Math.floor(x2*facteur);
    } else {
        ix1 = Math.ceil(x2*facteur);
        ix2 = Math.floor(x1*facteur);
    }
    for (var i=ix1; i<=ix2; i++) {
        if (x2 > x1)
            px = Math.round(((i/facteur - x1) / (x2 - x1)) * image.width) + wleft;
        else
            px = wleft + image.width - Math.round(((i/facteur - x2) / (x1 - x2)) * image.width);
        scoord = versString(i/facteur, 3);
        var metrics = context.measureText(scoord);
        context.fillText(scoord, px - metrics.width/2, h - hbottom/2);
        context.beginPath();
        context.strokeStyle = 'white';
        context.moveTo(px-0.5, 0);
        context.lineTo(px-0.5, h - hbottom);
        context.stroke();
        context.beginPath();
        context.strokeStyle = 'black';
        context.moveTo(px+0.5, 0);
        context.lineTo(px+0.5, h - hbottom);
        context.stroke();
    }
    context.restore();
}

function grilleY(facteur) {
    var iy1, iy2, py;
    var scoord;
    
    context.save();
    if (y2 > y1) {
        iy1 = Math.ceil(y1*facteur);
        iy2 = Math.floor(y2*facteur);
    } else {
        iy1 = Math.ceil(y2*facteur);
        iy2 = Math.floor(y1*facteur);
    }
    for (var i=iy1; i<=iy2; i++) {
        if (y2 > y1)
            py = Math.round(((i/facteur - y1) / (y2 - y1)) * (h - hbottom));
        else
            py = h - hbottom - Math.round(((i/facteur - y2) / (y1 - y2)) * (h - hbottom));
        scoord = versString(i/facteur, 3);
        context.fillText(scoord, 8, py + 2);
        context.beginPath();
        context.strokeStyle = 'white';
        context.moveTo(wleft, py-0.5);
        context.lineTo(wleft + image.width, py-0.5);
        context.stroke();
        context.beginPath();
        context.strokeStyle = 'black';
        context.moveTo(wleft, py+0.5);
        context.lineTo(wleft + image.width, py+0.5);
        context.stroke();
    }
    context.restore();
}

function coordonnees(e) {
    var x, y;
    if (e.pageX || e.pageY) {
      x = e.pageX;
      y = e.pageY;
    } else {
      x = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
      y = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
    }
    var totalOffsetX = 0;
    var totalOffsetY = 0;
    var currentElement = canvas;
    do {
        totalOffsetX += currentElement.offsetLeft;
        totalOffsetY += currentElement.offsetTop;
    } while (currentElement = currentElement.offsetParent)
    x -= totalOffsetX;
    y -= totalOffsetY;
    return( { x : x, y : y } );
}

function mousemove(e) {
    context.save();
    var pt = coordonnees(e);
    context.fillStyle = 'white';
    var metrics = context.measureText('-9.99999E-99, -9.99999E-99');
    context.fillRect(20, h - hbottom/2, metrics.width + 5, hbottom/2);
    if (pt.x < wleft + image.width && pt.x > wleft && pt.y < h - hbottom) {
        var x = ((pt.x - wleft) * 1.0 / image.width) * (x2 - x1) + x1;
        var y = (pt.y * 1.0 / (h - hbottom)) * (y2 - y1) + y1;
        var sx = versString(x, 5);
        var sy = versString(y, 5);
        context.fillStyle = 'black';
        if (labelx != null && labely != null)
            context.fillText(sx + ', ' + sy, 20, h - 5, 200);
        else if (labelx != null)
            context.fillText(sx, 20, h - 5, 200);
        else if (labely != null)
            context.fillText(sy, 20, h - 5, 200);
    }
    context.restore();
    if (e.preventDefault)
        e.preventDefault();
    else
        e.returnValue = false;
}

// fonction issue du Calcotron
function versString(v, precision) {
    var limitesci = 3; // décimales avant de passer en mode sci
    
    if (v == 0)
        return("0");
    var sj = ""+v;
    if ("Infinity" === sj || "-Infinity" === sj || "NaN" === sj)
        return(""+v);
    var signe;
    if (v >= 0)
        signe = "";
    else {
        v = -v;
        signe = "-";
    }
    var exposant = Math.floor(Math.log(v)/Math.log(10));
    var exparr = Math.pow(10, precision);
    var mantisse = v / Math.pow(10, exposant);
    mantisse = Math.round(mantisse*exparr)/exparr;
    if (mantisse >= 10) {
        exposant += 1;
        mantisse = v / Math.pow(10, exposant);
        mantisse = Math.round(mantisse*exparr)/exparr;
    }
    var sv = "";
    var chiffre = Math.floor(mantisse);
    sv += chiffre + ".";
    for (var i=0; i<precision-1; i++) {
        mantisse = (mantisse - chiffre)*10;
        var exparr2 = Math.pow(10, precision-i);
        mantisse = (Math.round(mantisse*exparr))/exparr;
        chiffre = Math.floor(mantisse);
        sv += chiffre;
    }
    mantisse = (mantisse - chiffre)*10;
    chiffre = Math.round(mantisse);
    sv += chiffre;
    var dernier = sv.charAt(sv.length-1);
    while (dernier == '0') {
        sv = sv.substring(0, sv.length-1);
        dernier = sv.charAt(sv.length-1);
    }
    if (exposant != 0) {
        if (exposant > 0 && exposant < limitesci) {
            var sd = sv.substring(0,1);
            if (sv.length-2 < exposant) {
                // transformation de 3.2E2 en 320
                if (sv.length > 2)
                    sd += sv.substring(2,sv.length);
                for (var i=0; i<exposant-(sv.length-2); i++)
                    sd += '0';
            } else {
                // transformation de 3.245E2 en 324.5
                if (sv.length < exposant+2)
                    sd += sv.substring(2);
                else
                    sd += sv.substring(2,exposant+2) + "." + sv.substring(exposant+2);
            }
            sv = sd;
        } else if (exposant < 0 && exposant > -limitesci) {
            // transformation de 3.2E-2 en 0.032
            var sd = "0.";
            for (var i=0; i<-exposant-1; i++)
                sd += '0';
            sv = sd + sv.charAt(0) + sv.substring(2);
        } else
            sv += "E" + exposant;
    }
    if (sv.charAt(sv.length-1) == '.')
        sv = sv.substring(0, sv.length-1);
    
    sv = signe + sv;
    
    return(sv);
}


